/**
 * @typedef {Object} ScreenshotsInfo
 *
 * A dictionary where each key contains a unique display identifier
 * and values are dictionaries with following items:
 * - id: Display identifier
 * - name: Display name, could be empty
 * - isDefault: Whether this display is the default one
 * - payload: The actual PNG screenshot data encoded to base64 string
 */
/**
 * Retrieves screenshots of each display available to Android.
 * This functionality is only supported since Android 10.
 *
 * @this {import('../driver').EspressoDriver}
 * @param {number|string} [displayId] Android display identifier to take a screenshot for.
 * If not provided then screenshots of all displays are going to be returned.
 * If no matches were found then an error is thrown.
 * @returns {Promise<ScreenshotsInfo>}
 */
export function mobileScreenshots(this: import("../driver").EspressoDriver, displayId?: number | string): Promise<ScreenshotsInfo>;
/**
 * Return the base 64 encoded screenshot data.
 * This method is called only when `appium:nativeWebScreenshot` is enabled
 * to avoid proxying requests to the chromedriver.
 * Without `appium:nativeWebScreenshot` or disabled, espresso driver
 * proxies screenshot endpoint requests to the espresso server directly.
 *
 * @this {import('../driver').EspressoDriver}
 * @returns {Promise<String>}
 */
export function getScreenshot(this: import("../driver").EspressoDriver): Promise<string>;
/**
 * A dictionary where each key contains a unique display identifier
 * and values are dictionaries with following items:
 * - id: Display identifier
 * - name: Display name, could be empty
 * - isDefault: Whether this display is the default one
 * - payload: The actual PNG screenshot data encoded to base64 string
 */
export type ScreenshotsInfo = any;
//# sourceMappingURL=screenshot.d.ts.map